function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import { Platform, NativeModules, NativeEventEmitter } from 'react-native';
import config from './config';
import IaphubError from './models/iaphub-error';
const LINKING_ERROR = `The package 'react-native-iaphub' doesn't seem to be linked. Make sure: \n\n` + Platform.select({
  ios: "- You have run 'pod install'\n",
  default: ''
}) + '- You rebuilt the app after installing the package\n' + '- You are not using Expo managed workflow\n';
const RNIaphub = NativeModules.RNIaphub ? NativeModules.RNIaphub : new Proxy({}, {
  get() {
    throw new Error(LINKING_ERROR);
  }

});
;
export default class Iaphub {
  constructor() {
    _defineProperty(this, "nativeEventEmitter", void 0);

    _defineProperty(this, "errorListener", void 0);

    _defineProperty(this, "listeners", []);

    this.nativeEventEmitter = new NativeEventEmitter(NativeModules.RNIaphub);
    this.listeners = [];
  }
  /**
   * Add event listener
   */


  addEventListener(name, callback) {
    const subscription = this.nativeEventEmitter.addListener(name, data => {
      if (name == "onError") {
        data = new IaphubError(data);
      }

      callback(data);
    });
    this.listeners.push(subscription);
    return subscription;
  }
  /**
   * Remove event listener
   */


  removeEventListener(listener) {
    var index = this.listeners.indexOf(listener);
    if (index === -1) return false;
    this.listeners[index].remove();
    this.listeners.splice(index, 1);
    return true;
  }
  /**
   * Remove all listeners
   */


  removeAllListeners() {
    this.listeners.forEach(listener => listener.remove());
    this.listeners = [];
  }
  /**
   * Start Iaphub
   * @param {String} appId App id that can be found on the IAPHUB dashboard
   * @param {String} apiKey Api key that can be found on the IAPHUB dashboard
   * @param {Boolean} allowAnonymousPurchase Option to allow purchases without being logged in
   * @param {Boolean} enableDeferredPurchaseListener Option to enable the onDeferredPurchase event (true by default)
   * @param {String} environment Option to specify a different environment than production
   * @returns {Promise<void>}
   */


  async start(opts) {
    try {
      // Build sdk version (we dot not define it in StartOptions on purpose, it is private)
      var sdkVersion = config.version;

      if (opts["sdkVersion"]) {
        sdkVersion += "/" + opts["sdkVersion"];
      } // Clear listeners


      this.removeAllListeners(); // Start IAPHUB

      await RNIaphub.start(Object.assign(opts, {
        sdkVersion: sdkVersion
      })); // Display product missing error

      this.errorListener = this.nativeEventEmitter.addListener("onError", err => {
        if (err.code == "unexpected" && err.subcode == "product_missing_from_store") {
          console.error(err.message);
        }
      });
    } catch (err) {
      throw IaphubError.parse(err);
    }
  }
  /**
   * Stop Iaphub
   * @returns {Promise<void>}
   */


  async stop() {
    try {
      // Clear listeners
      this.removeAllListeners(); // Stop IAPHUB

      await RNIaphub.stop(); // Remove error listener

      if (this.errorListener) {
        this.errorListener.remove();
      }
    } catch (err) {
      throw IaphubError.parse(err);
    }
  }
  /**
   * Log in user
   * @param {String} userId User id
   * @returns {Promise<void>}
   */


  async login(userId) {
    try {
      await RNIaphub.login(userId);
    } catch (err) {
      throw IaphubError.parse(err);
    }
  }
  /**
   * Ger user id
   * @returns {Promise<string>}
   */


  async getUserId() {
    try {
      var userId = await RNIaphub.getUserId();
      return userId;
    } catch (err) {
      throw IaphubError.parse(err);
    }
  }
  /**
   * Log out user
   * @returns {Promise<void>}
   */


  async logout() {
    try {
      await RNIaphub.logout();
    } catch (err) {
      throw IaphubError.parse(err);
    }
  }
  /**
   * Set device params
   * @param {Dict} params Device params
   * @returns {Promise<void>}
   */


  async setDeviceParams(params) {
    try {
      await RNIaphub.setDeviceParams(params);
    } catch (err) {
      throw IaphubError.parse(err);
    }
  }
  /**
   * Set user tags
   * @param {Dict} tags User tags
   * @returns {Promise<void>}
   */


  async setUserTags(tags) {
    try {
      await RNIaphub.setUserTags(tags);
    } catch (err) {
      throw IaphubError.parse(err);
    }
  }
  /**
   * Buy product
   * @param {String} sku Product sku
   * @param {Boolean} [crossPlatformConflict=true] Throws an error if the user has already a subscription on a different platform
   * @returns {Promise<Transaction>}
   */


  async buy(sku) {
    let opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {
      crossPlatformConflict: true
    };

    try {
      var transaction = await RNIaphub.buy(sku, opts);
      return transaction;
    } catch (err) {
      throw IaphubError.parse(err);
    }
  }
  /**
   * Restore purchases
   * @returns {Promise<RestoreResponse>}
   */


  async restore() {
    try {
      var response = await RNIaphub.restore();
      return response;
    } catch (err) {
      throw IaphubError.parse(err);
    }
  }
  /**
   * Get active products
   * @param {String[]} [includeSubscriptionStates=[]] Include subscription states (only 'active' and 'grace_period' states are returned by default)
   * @returns {Promise<ActiveProduct[]>}
   */


  async getActiveProducts() {
    let opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {
      includeSubscriptionStates: []
    };

    try {
      var products = await RNIaphub.getActiveProducts(opts);
      return products;
    } catch (err) {
      throw IaphubError.parse(err);
    }
  }
  /**
   * Get products for sale
   * @returns {Promise<Product[]>}
   */


  async getProductsForSale() {
    try {
      var products = await RNIaphub.getProductsForSale();
      return products;
    } catch (err) {
      throw IaphubError.parse(err);
    }
  }
  /**
   * Get products (active and for sale)
   * @returns {Promise<Products>}
   */


  async getProducts() {
    let opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {
      includeSubscriptionStates: []
    };

    try {
      var products = await RNIaphub.getProducts(opts);
      return products;
    } catch (err) {
      throw IaphubError.parse(err);
    }
  }
  /**
   * Get billing status
   * @returns {Promise<BillingStatus>}
   */


  async getBillingStatus() {
    try {
      var status = await RNIaphub.getBillingStatus();
      return status;
    } catch (err) {
      throw IaphubError.parse(err);
    }
  }
  /**
   * Present code redemption (iOS only)
   * @returns {Promise<void>}
   */


  async presentCodeRedemptionSheet() {
    try {
      await RNIaphub.presentCodeRedemptionSheet();
    } catch (err) {
      throw IaphubError.parse(err);
    }
  }
  /**
   * Show manage subscriptions page
   * @returns {Promise<void>}
   */


  async showManageSubscriptions() {
    let opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {
      sku: undefined
    };

    try {
      await RNIaphub.showManageSubscriptions(opts);
    } catch (err) {
      throw IaphubError.parse(err);
    }
  }

}
//# sourceMappingURL=iaphub.js.map