import { NativeEventEmitter, EmitterSubscription } from 'react-native';
import type Transaction from './models/transaction';
import type Product from './models/product';
import type ActiveProduct from './models/active-product';
import type BillingStatus from './models/billing-status';
declare type Products = {
    productsForSale: Product[];
    activeProducts: ActiveProduct[];
};
declare type RestoreResponse = {
    newPurchases: Transaction[];
    transferredActiveProducts: ActiveProduct[];
};
declare type EventName = 'onUserUpdate' | 'onDeferredPurchase' | 'onError' | 'onBuyRequest' | 'onReceipt';
interface StartOptions {
    appId: string;
    apiKey: string;
    userId?: string;
    allowAnonymousPurchase?: boolean;
    enableDeferredPurchaseListener?: boolean;
    environment?: string;
}
interface BuyOptions {
    crossPlatformConflict: boolean;
    prorationMode?: string;
}
interface GetProductsOptions {
    includeSubscriptionStates: string[];
}
interface ShowManageSubscriptionsOptions {
    sku?: string;
}
export default class Iaphub {
    nativeEventEmitter: NativeEventEmitter;
    errorListener?: EmitterSubscription;
    listeners: EmitterSubscription[];
    constructor();
    /**
     * Add event listener
     */
    addEventListener(name: EventName, callback: (data: any) => void): EmitterSubscription;
    /**
     * Remove event listener
     */
    removeEventListener(listener: EmitterSubscription): boolean;
    /**
     * Remove all listeners
     */
    removeAllListeners(): void;
    /**
     * Start Iaphub
     * @param {String} appId App id that can be found on the IAPHUB dashboard
     * @param {String} apiKey Api key that can be found on the IAPHUB dashboard
     * @param {Boolean} allowAnonymousPurchase Option to allow purchases without being logged in
     * @param {Boolean} enableDeferredPurchaseListener Option to enable the onDeferredPurchase event (true by default)
     * @param {String} environment Option to specify a different environment than production
     * @returns {Promise<void>}
     */
    start(opts: StartOptions): Promise<void>;
    /**
     * Stop Iaphub
     * @returns {Promise<void>}
     */
    stop(): Promise<void>;
    /**
     * Log in user
     * @param {String} userId User id
     * @returns {Promise<void>}
     */
    login(userId: string): Promise<void>;
    /**
     * Ger user id
     * @returns {Promise<string>}
     */
    getUserId(): Promise<string>;
    /**
     * Log out user
     * @returns {Promise<void>}
     */
    logout(): Promise<void>;
    /**
     * Set device params
     * @param {Dict} params Device params
     * @returns {Promise<void>}
     */
    setDeviceParams(params: {
        [key: string]: any;
    }): Promise<void>;
    /**
     * Set user tags
     * @param {Dict} tags User tags
     * @returns {Promise<void>}
     */
    setUserTags(tags: {
        [key: string]: any;
    }): Promise<void>;
    /**
     * Buy product
     * @param {String} sku Product sku
     * @param {Boolean} [crossPlatformConflict=true] Throws an error if the user has already a subscription on a different platform
     * @returns {Promise<Transaction>}
     */
    buy(sku: string, opts?: BuyOptions): Promise<Transaction>;
    /**
     * Restore purchases
     * @returns {Promise<RestoreResponse>}
     */
    restore(): Promise<RestoreResponse>;
    /**
     * Get active products
     * @param {String[]} [includeSubscriptionStates=[]] Include subscription states (only 'active' and 'grace_period' states are returned by default)
     * @returns {Promise<ActiveProduct[]>}
     */
    getActiveProducts(opts?: GetProductsOptions): Promise<ActiveProduct[]>;
    /**
     * Get products for sale
     * @returns {Promise<Product[]>}
     */
    getProductsForSale(): Promise<Product[]>;
    /**
     * Get products (active and for sale)
     * @returns {Promise<Products>}
     */
    getProducts(opts?: GetProductsOptions): Promise<Products>;
    /**
     * Get billing status
     * @returns {Promise<BillingStatus>}
     */
    getBillingStatus(): Promise<BillingStatus>;
    /**
     * Present code redemption (iOS only)
     * @returns {Promise<void>}
     */
    presentCodeRedemptionSheet(): Promise<void>;
    /**
     * Show manage subscriptions page
     * @returns {Promise<void>}
     */
    showManageSubscriptions(opts?: ShowManageSubscriptionsOptions): Promise<void>;
}
export {};
